/*
 * Copyright (c) 2001 University of Bologna
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill;

import java.io.*;

/**
 *  Interface AntView contains the set of methods which are
 *  accessible to ants. These methods enable ants to cooperate
 *  with anthills in a controlled way. For example, ants may
 *  retrieve resources from or add resources to an anthill;
 *  request routing information about other sites; move to
 *  other anthills, and so on.
 *
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public interface AntView
{

  /**
   *  Returns the identifier of the anthill containing the
   *  ant that uses this <CODE>AntView</CODE>.
   */
  public AnthillId getId();

  
  /**
   *  Returns the ant which is associated to this AntView. 
   */
  public Ant getAnt();
  
  
  /**
   *  Method invoked by an ant in order to add the identifier of a new 
   *  neighbour anthill to the set of neighbours known to the
   *  current anthill.
   */
  public void addNeighbour(AnthillId id);

  
  /**
   *  Method invoked by an ant in order to obtain routing information
   *  stored in the anthill.
   */
  public Neighbour[] getNeighbours();

    
  /**
   *  Method invoked by an ant in order to be sent to another 
   *  Anthill. It is possible that particular AntView implementation
   *  limit the operations which can be performed after a move
   *  operation. For example, an implementation may avoid that
   *  an anthill can move to more than one anthill (thus duplicating
   *  itself).
   *
   *  @param id the id of the destination anthill
   */
  public void move(AnthillId id)
    throws IOException;

  
  /**
   *  Performs the specified query on the local resource
   *  database and returns the resources satisfying the
   *  query. The interpretation of the query string 
   *  depends on the implementatation of the anthill.
   *
   *  @param query The query to be performed
   */
  public Resource[] getResources(String query);

  
  /**
   *  Add a resource to the anthill. This method enables
   *  ans to move resources from one anthill to another. 
   *  <p>
   *  Depending on the implementation of Resource, adding
   *  a Resource to an Anthill through an AntView may
   *  mean to actual copy the content of the resource,
   *  or to simply insert a reference to the actual 
   *  position of the resource itself. 
   *  <p>
   *  This method may be invoked more than once from an 
   *  ant, in order to insert more resources. Nevertheless,
   *  the actual insertion of a resource in an Anthill
   *  depends on the policies of the particular
   *  Anthill implementation. For example, an anthill
   *  may limit the number of resources originated by
   *  another anthill, in order to avoid "spamming".
   *
   *  @param resource resource to be added
   */
  public void addResource(Resource resource);

  
  /**
   *  Method invoked by an ant to notify the anthill that
   *  this ant, created to satisfy the given query, has 
   *  returned the specified set of resources.
   *
   *  @param query      the performed query
   *  @param resources  the resources satisfying the query
   */
  public void returnResources(String query, Resource[] resources);

  
} // END AntView
