/*
 * Copyright (c) 2001 University of Bologna
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill;

/**
 *  Class <CODE>Neighbour</CODE> is a container for the information
 *  that is stored in anthill and is related to other anthills.
 *  For each neighbour, information about the anthill identifier,
 *  the communication cost and the reachability status are
 *  maintained. In the case a <CODE>Neighbour</CODE> object is used by
 *  an <CODE>Ant</CODE>, information about the pheromone track is also 
 *  stored and may be manipulated accordingly.
 *  <p>
 *  This class is designed to be used by ants. For this reasons, it
 *  does not include mutator methods for costs and reachability status, 
 *  which are to be evaluated by the anthill hosting the ant. Anthill 
 *  implementations may extend this class to provide package-accessible 
 *  methods for mutating those values. Access to those methods will
 *  be restricted through access control.
 *
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class Neighbour
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Identifier of the anthill neighbour */
  private AnthillId id;
  
  /** Communication cost with this neighbour */
  private float cost;

  /** True if this neighbour is currently reachable */
  private boolean reachable;

  /** Pheromone object managed by ants */
  private Object pheromone;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Constructs a new neighbour with the given identifier.
   */
  public Neighbour(AnthillId id)
  {
    this.id = id;
    this.cost = 0;
    this.reachable = true;
  }
  
  /**
   *  Constructs a new neighbour with the given identifier.
   */
  public Neighbour(AnthillId id, float cost, boolean reachable)
  {
    this.id = id;
    this.cost = cost;
    this.reachable = reachable;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns the anthill identifier of the neighbour 
   */
  public AnthillId getId()
  {
    return id;
  }
  
  
  /**
   *  Returns a float value representing the cost of
   *  communication with this neighbor.
   */
  public float getCost()
  {
    return cost;
  }

  
  /**
   *  Returns true if this neghbour is considered reachable.
   */
  public boolean isReachable()
  {
    return reachable;
  }
  
  
  /**
   *  Return the pheromone associated to the link to this
   *  neighbour. The class of the pheromone is left unspecified.
   */
  public Object getPheromone()
  {
    return pheromone;
  }

  
  /**
   *  Return the pheromone associated to the link to this
   *  neighbour. The class of the pheromone is left unspecified.
   */
  public void setPheromone(Object pheromone)
  {
    this.pheromone = pheromone;
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Object
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Check whether this <CODE>LocalAid</CODE> is equal to the specified object.
   */
  public boolean equals(Object obj)
  {
    if (obj != null && obj instanceof Neighbour) {
      Neighbour neighbour = (Neighbour) obj;
      return (this.id == neighbour.id);
    } else {
      return false;
    }
  }

  /**
   *  Returns a hash code value for the object.
   */
  public int hashCode()
  {
    return id.hashCode();
  }
  
  /**
   *  Returns a string representation of the object.
   */
  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    buf.append("[");
    buf.append(this.getClass().getName());
    buf.append(": ");
    buf.append(id.toString());
    buf.append(", ");
    buf.append(cost);
    buf.append(", ");
    buf.append(reachable);
    buf.append("]");
    return buf.toString();
  }

  
  
} // END Neighbour

