/*
 * Copyright (c) 2001 University of Bologna
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.antnet;

import java.util.*;
import anthill.*;
import anthill.util.*;

/**
 *  Implementations of Manager capable to manage multiple
 *  threads.
 *
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class ConcurrentManager
implements Manager
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Queue containing the agents to be computed */
  Queue queue;
  
  /** Number of threads in the active set */
  int nthreads;
  
  /** Free threads */
  HashSet free;
  
  /** Busy threads */
  HashSet busy;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Instantiates a thread manager by creating a number of
   *  threads which will take care of the agent.
   */
  public ConcurrentManager(int nthreads)
  {
    this.nthreads = nthreads;
    
    free = new HashSet();
    busy = new HashSet();
    
    for (int i=0; i < nthreads; i++) {
      Executor e = new Executor(this);
      free.add(e);
    }
    
    queue = new Queue();
      
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Manager
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Adds an agent descriptor to the queue of agents to be 
   *  executed
   *
   *  @param desc the agent descriptor to be added
   */
  public synchronized void add(AntView view)
  {
    if (!free.isEmpty()) {
      /*
       *  There are available threads; remove the first thread
       *  from the free set, add it to the busy set, and assign
       *  it to the specified agent.
       */
      Executor e = (Executor) free.iterator().next();
      free.remove(e);
      busy.add(e);
      e.execute(view);
    } else {
      /*
       *  No free thread is available; add the agent to the queue
       *  of agents to be executed.
       */
      queue.add(view);
    }
  }
  
 
  /**
   *  Execute an agent maintened in this Manager
   */
  public void execute()
  {
    throw new UnsupportedOperationException();
  }
  
  
  /**
   *  Receives the notification that a thread handled by this
   *  thread manager has completed its agent, and its ready
   *  to deal with another agent.
   */
  synchronized void completed(Thread thread)
  {
    Executor executor;
    try {
      executor = (Executor) thread;
    } catch (ClassCastException e) {
      /*
       *  The thread for which we got a completed() invocation
       *  wasn't a Executor thread,, so some critical error has 
       *  occured. To avoid problems, we panic.
       */
      System.err.println("ConcurrentManager.completed(): " +
        "thread is not an Executor: " + thread.getClass().getName()); 
      System.exit(0);
      return;
    }
    if (busy.contains(executor)) {
      /*
       *  The thread for which we got a completed() invocation 
       *  was actually one of our threads
       */
      if (queue.isEmpty()) {
        /* No agent is waiting, so we insert the thread into the free list */
        busy.remove(executor);
        free.add(executor);
      } else {
        /* We assign a new agent immediately */
        AntView view = (AntView) queue.remove();
        executor.execute(view);
      }
    } else {
      /*
       *  The thread for which we got a completed() invocation
       *  wasn't in the busy set, so some cirtical error has 
       *  occured. To avoid problems, we panic.
       */
      System.err.println("ConcurrentManager.completed(): thread not existing"); 
      System.exit(0);
      return;
    }
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Manager
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Restore the object to its initial state.
   */
  public void clear()
  {
    // FIXME Complete this method
  }
  
  
} // END ConcurrentManager
