/*
 * Copyright (c) 2001 University of Bologna
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.genie;

import anthill.*;
import anthill.gf.*;
import anthill.util.*;

/**
 *  Class ExperimentImpl is a simple implementation of Experiment.
 *  It is constituted by a set of scenarios, on which the fitness
 *  is computed as the average fitness as computed in each scenario.
 *
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class ExperimentImpl
implements Experiment
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Reference to the experiment factory which created this object */
  private ExperimentFactoryImpl experimentf;

  /** Number of scenarios to be evaluated  */
  private int size;
  
  /** Arrays of scenarios on which to evaluate the genetic code */
  private Scenario[] scenarios;
  
  /** Genetic code to be evaluated */
  private GeneticCode code;
  
  /** Evaluated fitness */
  private double fitness;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Instantiates a new experiment by creating the specified
   *  number of scenarios through the specified scenario factory.
   *
   */
  public ExperimentImpl(ExperimentFactoryImpl experimentf)
  {
    this.experimentf = experimentf;
    size = experimentf.getSize();
    code = null;
    fitness = -1.0;
    scenarios = new Scenario[size];
    ScenarioFactory scenariof = experimentf.getScenarioFactory();
    for (int i=0; i < size; i++) 
      scenarios[i] = scenariof.getScenario();
  }
  
   
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Experiment
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Set the genetic code to be evaluated. In this basic implementation,
   *  only one genetic code can be present in the Anthill network at a time.
   *  If this method is called more than once, exception IllegalStateException
   *  is thrown.
   */
  public void addGeneticCode(GeneticCode code)
  {
    if (this.code != null)
      throw new IllegalStateException("ExperimentImpl can evaluate only one genetic code");
    this.code = code;
  }

  /**
   *  Execute the experiment, i.e. evaluate the fitness of the genetic code on
   *  each of the scenarios contained in this experimnt. The fitness of the
   *  genetic code for this experiment is obtained by the sum of the fitness 
   *  values obtained by evaluating them on the scenarios.
   *
   *  @throws IllegalStateException if no genetic code is present
   */
  public void evaluate()
  {
    if (this.code == null)
      throw new IllegalStateException("No genetic code to be evaluated");
    
    fitness = 0.0;
    
    for (int i=0; i < size; i++) {
      scenarios[i].addGeneticCode(code);
      fitness += scenarios[i].evaluate();
    }

    fitness = fitness / size;
    Log.println(200, "Genetic code: " + code + " found " + fitness + "% resources");
  }

  /**
   *  Return the evaluated fitness. This operation is separated from
   *  the actual evaluation in order to enable the computation of
   *  fitness values using "resource sharing" techniques.
   *
   *  @throws IllegalStateException if the fitness value has not been computed
   *    yet.
   */
  public double getFitness()
  {
    if (fitness < 0)
      throw new IllegalStateException("No fitness value yet!");
    return fitness;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Clearable
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Restore the object to its initial state. Method clear() is invoked on
   *  all scenarios included in this experiment. The genetic code to be
   *  evaluated and the computed fitness are restored to their initial
   *  state.
   */
  public void clear()
  {
    code = null;
    fitness = -1.0;
    for (int i=0; i < size; i++)
      scenarios[i].clear();
  }

} // END ExperimentImpl

