/*
 * Copyright (c) 2001 University of Bologna
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.genie;

import anthill.*;
import anthill.gf.*;

import java.util.Random;

/**
 *  This is a basic implementation of the Peernet interface. PeernetImpl 
 *  objects can be created by using a BasicPeernetFactory, or using a 
 *  SingletonPeernetFactory.
 *  
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class PeernetImpl
implements Peernet
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static section
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Random number generator */
  private static Random random = new Random();
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Reference to the peernet factory which created this object*/
  private PeernetFactoryImpl peernetf;
  
  /** Anthill network */
  Anthill[] nodes;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Initialize a new Peernet factory
   */
  public PeernetImpl(PeernetFactoryImpl peernetf)
  {
    this.peernetf = peernetf;
    init(true);
  }
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Peernet
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Return the nodes composing the simulated p2p network
   */
  public Anthill[] getNodes()
  {
    return nodes;
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Clearable
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Restore the object to its initial state. The network is re-initialized
   *  as an empty network.
   */
  public void clear()
  {
    init(false);
  }
    
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Private methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Initialize the object
   *
   *  @param constructor true if the method is invoked by the
   *    constructor
   */
  private void init(boolean constructor)
  {
    int size = peernetf.getSize();
    int degree = peernetf.getDegree();

    /*
     *  Creates or clears the nodes of the peernet;
     *  new nodes are created if we are in the constructor,
     *  or if the peernet factory required a size different
     *  from the length of the node array
     */
    if (constructor || size != nodes.length) {
      nodes = new Anthill[size];
      for (int i=0; i < size; i++) {
        nodes[i] = peernetf.getAnthillFactory().getAnthill();
      }
    } else {
      for (int i=0; i < size; i++) {
        nodes[i].clear();
      }
    }
      
    /*
     *  Create arcs (in both cases)
     */
    for (int i=0; i < size; i++) {
      for (int j=0; j < degree; j++) {
        int k = random.nextInt(size);
        nodes[k].addClientNeighbour(nodes[i].getId());
        nodes[i].addClientNeighbour(nodes[k].getId());
      }
    }
    
    /*
     *  FIXME: Verify that the created graph is connected
     */
  }

} // END PeernetImpl

