/*
 * Copyright (c) 2001 University of Bologna
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.gf;

import anthill.util.*;

/**
 *
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class Statistics
implements Clearable
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Number of ant moves */
  private int antMoveCounter;

  /** Cost of ant moves */
  private float antMoveCost;

  /** Number of resources added by ants */
  private int antTransferResourcesCounter;

  /** Cost of adding resources on behalf of ants */
  private float antTransferResourcesCost;

  /** Number of resources returned by ants */
  private int clientTransferResourcesCounter;

  /** Cost of returning resources on behalf of ants */
  private float clientTransferResourcesCost;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Initializes all recorded fields to zero.
   */
  public Statistics()
  {
    init(true);
  }
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Increment the total number of ant moves and add the specified
   *  cost to the total cost of ant moves.
   *  <p>
   *  This method should be invoked to record the cost of moving an 
   *  ant from an anthill to another, and not the cost of moving 
   *  resources which may be transferred together with a moving ant. 
   *  The costs of moving resources should be recorded through 
   *  <CODE>incAntTransferResources()</CODE>.
   *
   *  @param cost cost to be added
   */
  public void incAntMove(float cost)
  {
    antMoveCounter++;
    antMoveCost += cost;
  }

  /**
   *  Return the total number of ant moves.
   */
  public int getAntMoveCounter()
  {
    return antMoveCounter;
  }

  /**
   *  Return the total cost of ant moves.
   */
  public float getAntMoveCost()
  {
    return antMoveCost;
  }

  
  /**
   *  Increment the number of resources added by ants and the 
   *  total cost of these addings (i.e., the cost of transferring 
   *  resources from one anthill to another).
   *  <p>
   *  This method should be invoked to record the cost of cloning 
   *  resources from an anthill to another on behalf of ants, and 
   *  not the cost of transferring a resource from an anthill to 
   *  the client which requested it. The costs of transferring
   *  resources on behalf of clients should be recorded through 
   *  <CODE>incClientTransferResourcess()</CODE>.
   *
   *  @param cost cost of adding the resource
   */
  public void incAntTransferResources(float cost)
  {
    antTransferResourcesCounter++;
    antTransferResourcesCost += cost;
  }

  /**
   *  Return the total number of resources added by ants.
   */
  public int getAntTransferResourcesCounter()
  {
    return antTransferResourcesCounter;
  }


  /**
   *  Return the total cost of adding resources on behalf
   *  of ants.
   */
  public float getAntTransferResourcesCost()
  {
    return antTransferResourcesCost;
  }

  
  /**
   *  Increment the number of resources returned by ants and the 
   *  total cost of these addings (i.e., the cost of transferring 
   *  these resources from one anthill to the client).
   *  <p>
   *  This method should be invoked to record the cost of transferring
   *  resources from an anthill to a client. The cost of returning
   *  resources which are found locally should not be counted. For
   *  these costs, method <CODE>incLocalReturnResorces()</CODE>
   *  should be used.
   *
   *  @param cost cost of transferring a resource on behalf of
   *   the client
   */
  public void incClientTransferResources(float cost)
  {
    clientTransferResourcesCounter++;
    clientTransferResourcesCost += cost;
  }

  /**
   *  Return the total number of resources returned to clients.
   */
  public int getClientTransferResourcesCounter()
  {
    return clientTransferResourcesCounter;
  }

  /**
   *  Return the total cost of returning resources to clients.
   */
  public float getClientTransferResourcesCost()
  {
    return clientTransferResourcesCost;
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Clearable
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Restore the object to its initial state, setting all fields
   *  to zero.
   */
  public void clear()
  {
    init(false);
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Private methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Initialize the object
   *
   *  @param constructor true if the method is invoked by the
   *    constructor
   */
  private void init(boolean constructor)
  {
    antMoveCounter = 0;
    antMoveCost = 0;
    antTransferResourcesCost = 0;
    antTransferResourcesCost = 0;
  }

} // END Statistics

