/*
 * Copyright (c) 2001 University of Bologna
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.nestor;

import anthill.*;
import anthill.util.Printable;

/**
 *  NestorAnthillFactory is the factory class for NestorAnthill
 *  objects. In order to build an anthill, you need (i)
 *  a storage factory to obtain a storage object for resources;
 *  (ii) a gate factory to obtain a gate object for communicating
 *  with other anthills; (iii) a manager factory to obtain a
 *  manager object for managing incoming ants; (iv) a view
 *  factory to build view objects to be presented to ants;
 *  a ttl parameter to avoid "immortal ants". All these 
 *  configuration parameters can be obtained by an
 *  NestorAnthillFactory by invoking the opportune get
 *  method. 
 *
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class NestorAnthillFactory
extends Printable
implements   AnthillFactory
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Storage factory; used only once when instantiating an anthill */
  private StorageFactory storagef;

  /** Gate factory; used only once when instantiating an anthill */
  private GateFactory gatef;
  
  /** Manager factory; used only once when instantiating an anthill */
  private ManagerFactory managerf;

  /** QuerySet factory; used only once when instantiating an anthill */
  private QuerySetFactory querysetf;

  /** Ant factory */
  private AntFactory antf;

  /** AntView factory */
  private AntViewFactory viewf;

  /** Default time-to-live */
  private int ttl;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Basic constructor whose task is to store the initialization
   *  parameters.
   */
  public NestorAnthillFactory(StorageFactory storagef, GateFactory gatef, 
    ManagerFactory managerf, QuerySetFactory querysetf, AntFactory antf, 
    AntViewFactory viewf, int ttl)
  {
    this.storagef = storagef;
    this.gatef = gatef;
    this.managerf = managerf;
    this.querysetf = querysetf;
    this.antf = antf;
    this.viewf = viewf;
    this.ttl = ttl;
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from AnthillFactory interface
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Return a new anthill based on the initialization parameters
   *  stored in this anthill factory. A reference to this factory
   *  is passed to the anthill constructor, in order that the 
   *  anthill could obtain the initialization parameters.
   */
  public Anthill getAnthill()
  {
    return new NestorAnthill(this);
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // "getter" methods for fields (package visibility)
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Return the storage factory included in this object
   */
  final StorageFactory getStorageFactory()
  {
    return storagef;
  }
  
  /**
   *  Return the gate factory included in this object
   */
  final GateFactory getGateFactory()
  {
    return gatef;
  }
  
  /**
   *  Return the storage factory included in this object
   */
  final ManagerFactory getManagerFactory()
  {
    return managerf;
  }

  /**
   *  Return the query set factory included in this object
   */
  final QuerySetFactory getQuerySetFactory()
  {
    return querysetf;
  }

  /**
   *  Return the storage factory included in this object
   */
  final AntFactory getAntFactory()
  {
    return antf;
  }

  /**
   *  Return the storage factory included in this object
   */
  final AntViewFactory getAntViewFactory()
  {
    return viewf;
  }

  /**
   *  Return the storage factory included in this object
   */
  final int getTTL()
  {
    return ttl;
  }
  
} // END NestorAnthillFactory

